@@ -1,6 +1,6 @@
 # pROC: Tools Receiver operating characteristic (ROC curves) with
 # (partial) area under the curve, confidence intervals and comparison. 
-# Copyright (C) 2010, 2011 Xavier Robin, Alexandre Hainard, Natacha Turck,
+# Copyright (C) 2010-2014 Xavier Robin, Alexandre Hainard, Natacha Turck,
 # Natalia Tiberti, Frédérique Lisacek, Jean-Charles Sanchez
 # and Markus Müller
 #
@@ -22,26 +22,48 @@ roc <- function(...) {
 }
 
 roc.formula <- function (formula, data, ...){
-  cl <- match.call()
+  # Get predictors (easy)
+  predictors <- attr(terms(formula), "term.labels")
+  
+  # Get the data (complicated)
   m <- match.call(expand.dots = FALSE)
   if (is.matrix(eval(m$data, parent.frame())))
     m$data <- as.data.frame(data)
   m$... <- NULL
   m[[1]] <- as.name("model.frame")
   m <- eval(m, parent.frame())
-  term.labels <- attr(attr(m, "terms"), "term.labels")
-  response <- model.extract(m, "response")
+  
+  # Get response (easy)
+  response <- model.response(m)
 
-  if (length(term.labels) != 1) {
-    stop("Invalid formula: exactly 1 predictor is required in a formula of type response~predictor.")
-  }
   if (length(response) == 0) {
     stop("Error in the formula: a response is required in a formula of type response~predictor.")
   }
 
-  roc <- roc.default(response, m[[term.labels]], ...)
-  roc$call <- match.call()
-  return(roc)
+  if (length(predictors) == 1) {
+    roc <- roc.default(response, m[[predictors]], ...)
+    roc$call <- match.call()
+    if (!is.null(roc$smooth))
+      attr(roc, "roc")$call <- roc$call
+    return(roc)
+  }
+  else if (length(predictors) > 1) {
+    roclist <- lapply(predictors, function(predictor, formula, data, call, ...) {
+      # Get one ROC
+      roc <- roc.default(response, data[[predictor]], ...)
+      # Update the call to reflect the parents
+      formula[3] <- call(predictor) # replace the predictor in formula
+      call$formula <- formula # Replace modified formula
+      roc$call <- call
+      return(roc)
+    }, formula = formula, data = m, call = match.call(), ...)
+    # Set the list names
+    names(roclist) <- predictors
+    return(roclist)
+  }
+  else {
+    stop("Invalid formula:at least 1 predictor is required in a formula of type response~predictor.")
+  }
 }
 
 roc.default <- function(response, predictor,
@@ -185,63 +207,122 @@ roc.default <- function(response, predictor,
     stop("No valid data provided.")
   }
 
-  # create the roc object
-  roc <- list()
-  class(roc) <- "roc"
-  roc$levels <- levels
-  roc$percent <- percent
-
-  roc$call <- match.call()
-
   if (direction == "auto" && median(controls) <= median(cases))
     direction <- "<"
   else if (direction == "auto" && median(controls) > median(cases))
     direction <- ">"
-
-  # compute SE / SP
-  thresholds <- roc.utils.thresholds(c(controls, cases))
   
+  # smooth with densities, but only density was provided, not density.controls/cases
+  if (smooth) {
+    if (missing(density.controls))
+      density.controls <- density
+    if (missing(density.cases))
+      density.cases <- density
+  }
+  
+  # Choose algorithm
   if (identical(algorithm, 0)) {
     if (!require(microbenchmark))
       stop("Package microbenchmark not available, required with algorithm=0'. Please install it with 'install.packages(\"microbenchmark\")'.")
     cat("Starting benchmark of algorithms 2 and 3, 10 iterations...\n")
-    benchmark <- microbenchmark(
+    thresholds <- roc.utils.thresholds(c(controls, cases))
+    benchmark <- try(microbenchmark::microbenchmark(
       "2" = roc.utils.perfs.all.fast(thresholds=thresholds, controls=controls, cases=cases, direction=direction),
       "3" = rocUtilsPerfsAllC(thresholds=thresholds, controls=controls, cases=cases, direction=direction),
       times = 10
-      )
-    print(summary(benchmark))
-    roc$fun.sesp <- if (which.min(tapply(benchmark$time, benchmark$expr, sum)) == 1) {
-      cat("Selecting algorithm 2.\n")
-      roc.utils.perfs.all.fast
-    }
+    ))
+    if (is(benchmark, "try-error")) {
+      warning("Microbenchmark failed. Using default algorithm 1.")
+      algorithm <- 1
+    } 
     else {
-      cat("Selecting algorithm 3.\n")
-      rocUtilsPerfsAllC
+      print(summary(benchmark))
+      if (any(is.na(benchmark))) {
+        warning("Microbenchmark returned NA. Using default algorithm 1.")
+        algorithm <- 1
+      }
+      else if (which.min(tapply(benchmark$time, benchmark$expr, sum)) == 1) {
+        algorithm <- 2
+        cat("Selecting algorithm 2.\n")
+      }
+      else {
+        algorithm <- 3
+        cat("Selecting algorithm 3.\n")
+      }
     }
-    perfs <- roc$fun.sesp(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
   }
-  else if (identical(algorithm, 1)) {
-    roc$fun.sesp <- roc.utils.perfs.all.safe
-    perfs <- roc.utils.perfs.all.safe(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
-    
+  if (identical(algorithm, 1)) {
+    fun.sesp <- roc.utils.perfs.all.safe
   }
   else if (identical(algorithm, 2)) {
-    roc$fun.sesp <- roc.utils.perfs.all.fast
-    perfs <- roc.utils.perfs.all.fast(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+    fun.sesp <- roc.utils.perfs.all.fast
   }
   else if (identical(algorithm, 3)) {
-    roc$fun.sesp <- rocUtilsPerfsAllC
-    perfs <- rocUtilsPerfsAllC(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+    fun.sesp <- rocUtilsPerfsAllC
   }
   else if (identical(algorithm, 4)) {
-  	roc$fun.sesp <- roc.utils.perfs.all.test
-  	perfs <- roc.utils.perfs.all.test(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+    fun.sesp <- roc.utils.perfs.all.test
   }
   else {
-    stop("Unknown algorithm (must be 1, 2, 3 or 4).")
+    stop("Unknown algorithm (must be 0, 1, 2, 3 or 4).")
   }
 
+  roc <- roc.cc.nochecks(controls, cases,
+             percent=percent,
+             direction=direction,
+             fun.sesp=fun.sesp,
+             smooth = smooth, density.cases = density.cases,  density.controls = density.controls, smooth.method = smooth.method, 
+             auc, ...)
+  
+  roc$call <- match.call()
+  if (smooth) {
+    attr(roc, "roc")$call <- roc$call
+    attr(roc, "roc")$original.predictor <- original.predictor
+    attr(roc, "roc")$original.response <- original.response
+    attr(roc, "roc")$predictor <- predictor
+    attr(roc, "roc")$response <- response
+    attr(roc, "roc")$levels <- levels
+  }
+  roc$original.predictor <- original.predictor
+  roc$original.response <- original.response
+  roc$predictor <- predictor
+  roc$response <- response
+  roc$levels <- levels
+  
+  # compute CI
+  if (ci)
+    roc$ci <- ci(roc, method=ci.method, ...)
+  # plot
+  if (plot)
+    plot.roc(roc, ...)
+  
+  # return roc
+  return(roc)
+}
+
+#' Creates a ROC object from response, predictor, ... without argument checking. Not to be exposed to the end user
+roc.rp.nochecks <- function(response, predictor, levels, ...) {
+  splitted <- split(predictor, response)
+  controls <- splitted[[as.character(levels[1])]]
+  if (length(controls) == 0)
+    stop("No control observation.")
+  cases <- splitted[[as.character(levels[2])]]
+  if (length(cases) == 0)
+    stop("No case observation.")
+  roc.cc.nochecks(controls, cases, ...)
+}
+
+#' Creates a ROC object from controls, cases, ... without argument checking. Not to be exposed to the end user
+roc.cc.nochecks <- function(controls, cases, percent, direction, fun.sesp, smooth, smooth.method, auc, ...) {
+  # create the roc object
+  roc <- list()
+  class(roc) <- "roc"
+  roc$percent <- percent
+
+  # compute SE / SP
+  thresholds <- roc.utils.thresholds(c(controls, cases))
+  perfs <- fun.sesp(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+
   se <- perfs$se
   sp <- perfs$sp
 
@@ -249,6 +330,7 @@ roc.default <- function(response, predictor,
     se <- se*100
     sp <- sp*100
   }
+
   # store the computations in the roc object
   roc$sensitivities <- se
   roc$specificities <- sp
@@ -257,31 +339,14 @@ roc.default <- function(response, predictor,
   roc$direction <- direction
   roc$cases <- cases
   roc$controls <- controls
-  roc$original.predictor <- original.predictor
-  roc$original.response <- original.response
-  roc$predictor <- predictor
-  roc$response <- response
-
-  # smooth?
+  roc$fun.sesp <- fun.sesp
+  
   if (smooth) {
-    if (missing(density.controls))
-      density.controls <- density
-    if (missing(density.cases))
-      density.cases <- density
-    roc <- smooth.roc(roc, density=density, density.controls=density.controls, density.cases=density.cases, method=smooth.method, ...)
-    roc$call <- match.call()
+    roc <- smooth.roc(roc, method=smooth.method, ...)
   }
-
   # compute AUC
   if (auc)
-    roc$auc <- auc(roc, ...)
-  # compute CI
-  if (ci)
-    roc$ci <- ci(roc, method=ci.method, ...)
-  # plot
-  if (plot)
-    plot.roc(roc, ...)
-
-  # return roc
+    roc$auc <- auc.roc(roc, ...)
+  
   return(roc)
 }
